"""initial_migration

Revision ID: d51c7172d2f2
Revises: 
Create Date: 2023-05-06 17:46:39.230122

"""
from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = 'd51c7172d2f2'
down_revision = None
branch_labels = None
depends_on = None


def upgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_table('Author',
    sa.Column('uid', sa.Integer(), nullable=False),
    sa.Column('name', sa.Text(), nullable=False),
    sa.PrimaryKeyConstraint('uid', name=op.f('pk_Author'))
    )
    with op.batch_alter_table('Author', schema=None) as batch_op:
        batch_op.create_index(batch_op.f('ix_Author_name'), ['name'], unique=True)

    op.create_table('Bookcase',
    sa.Column('description', sa.Text(), nullable=True),
    sa.Column('uid', sa.Integer(), nullable=False),
    sa.Column('name', sa.Text(), nullable=False),
    sa.PrimaryKeyConstraint('uid', name=op.f('pk_Bookcase'))
    )
    with op.batch_alter_table('Bookcase', schema=None) as batch_op:
        batch_op.create_index(batch_op.f('ix_Bookcase_name'), ['name'], unique=True)

    op.create_table('Category',
    sa.Column('description', sa.Text(), nullable=True),
    sa.Column('uid', sa.Integer(), nullable=False),
    sa.Column('name', sa.Text(), nullable=False),
    sa.PrimaryKeyConstraint('uid', name=op.f('pk_Category'))
    )
    with op.batch_alter_table('Category', schema=None) as batch_op:
        batch_op.create_index(batch_op.f('ix_Category_name'), ['name'], unique=True)

    op.create_table('Language',
    sa.Column('iso639_1_code', sa.String(length=2), nullable=False),
    sa.Column('uid', sa.Integer(), nullable=False),
    sa.Column('name', sa.Text(), nullable=False),
    sa.PrimaryKeyConstraint('uid', name=op.f('pk_Language'))
    )
    with op.batch_alter_table('Language', schema=None) as batch_op:
        batch_op.create_index(batch_op.f('ix_Language_iso639_1_code'), ['iso639_1_code'], unique=True)
        batch_op.create_index(batch_op.f('ix_Language_name'), ['name'], unique=True)

    op.create_table('MediaType',
    sa.Column('description', sa.Text(), nullable=True),
    sa.Column('uid', sa.Integer(), nullable=False),
    sa.Column('name', sa.Text(), nullable=False),
    sa.PrimaryKeyConstraint('uid', name=op.f('pk_MediaType'))
    )
    with op.batch_alter_table('MediaType', schema=None) as batch_op:
        batch_op.create_index(batch_op.f('ix_MediaType_name'), ['name'], unique=True)

    op.create_table('BookcaseShelf',
    sa.Column('description', sa.Text(), nullable=True),
    sa.Column('row', sa.SmallInteger(), nullable=False),
    sa.Column('column', sa.SmallInteger(), nullable=False),
    sa.Column('fk_bookcase_uid', sa.Integer(), nullable=False),
    sa.Column('uid', sa.Integer(), nullable=False),
    sa.ForeignKeyConstraint(['fk_bookcase_uid'], ['Bookcase.uid'], name=op.f('fk_BookcaseShelf_fk_bookcase_uid_Bookcase')),
    sa.PrimaryKeyConstraint('uid', name=op.f('pk_BookcaseShelf')),
    sa.UniqueConstraint('column', 'fk_bookcase_uid', 'row', name=op.f('uq_BookcaseShelf_column'))
    )
    op.create_table('BookcaseItem',
    sa.Column('isbn', sa.String(), nullable=False),
    sa.Column('owner', sa.String(), nullable=False),
    sa.Column('amount', sa.SmallInteger(), nullable=False),
    sa.Column('fk_media_type_uid', sa.Integer(), nullable=False),
    sa.Column('fk_bookcase_shelf_uid', sa.Integer(), nullable=True),
    sa.Column('fk_language_uid', sa.Integer(), nullable=True),
    sa.Column('uid', sa.Integer(), nullable=False),
    sa.Column('name', sa.Text(), nullable=False),
    sa.ForeignKeyConstraint(['fk_bookcase_shelf_uid'], ['BookcaseShelf.uid'], name=op.f('fk_BookcaseItem_fk_bookcase_shelf_uid_BookcaseShelf')),
    sa.ForeignKeyConstraint(['fk_language_uid'], ['Language.uid'], name=op.f('fk_BookcaseItem_fk_language_uid_Language')),
    sa.ForeignKeyConstraint(['fk_media_type_uid'], ['MediaType.uid'], name=op.f('fk_BookcaseItem_fk_media_type_uid_MediaType')),
    sa.PrimaryKeyConstraint('uid', name=op.f('pk_BookcaseItem'))
    )
    with op.batch_alter_table('BookcaseItem', schema=None) as batch_op:
        batch_op.create_index(batch_op.f('ix_BookcaseItem_isbn'), ['isbn'], unique=True)
        batch_op.create_index(batch_op.f('ix_BookcaseItem_name'), ['name'], unique=True)

    op.create_table('BookcaseItemBorrowing',
    sa.Column('username', sa.String(), nullable=False),
    sa.Column('start_time', sa.DateTime(), nullable=False),
    sa.Column('end_time', sa.DateTime(), nullable=False),
    sa.Column('delivered', sa.Boolean(), nullable=False),
    sa.Column('fk_bookcase_item_uid', sa.Integer(), nullable=False),
    sa.Column('uid', sa.Integer(), nullable=False),
    sa.ForeignKeyConstraint(['fk_bookcase_item_uid'], ['BookcaseItem.uid'], name=op.f('fk_BookcaseItemBorrowing_fk_bookcase_item_uid_BookcaseItem')),
    sa.PrimaryKeyConstraint('uid', name=op.f('pk_BookcaseItemBorrowing'))
    )
    with op.batch_alter_table('BookcaseItemBorrowing', schema=None) as batch_op:
        batch_op.create_index(batch_op.f('ix_BookcaseItemBorrowing_fk_bookcase_item_uid'), ['fk_bookcase_item_uid'], unique=False)

    op.create_table('BookcaseItemBorrowingQueue',
    sa.Column('username', sa.String(), nullable=False),
    sa.Column('entered_queue_time', sa.DateTime(), nullable=True),
    sa.Column('should_notify_user', sa.Boolean(), nullable=True),
    sa.Column('fk_bookcase_item_uid', sa.Integer(), nullable=False),
    sa.Column('uid', sa.Integer(), nullable=False),
    sa.ForeignKeyConstraint(['fk_bookcase_item_uid'], ['BookcaseItem.uid'], name=op.f('fk_BookcaseItemBorrowingQueue_fk_bookcase_item_uid_BookcaseItem')),
    sa.PrimaryKeyConstraint('uid', name=op.f('pk_BookcaseItemBorrowingQueue'))
    )
    with op.batch_alter_table('BookcaseItemBorrowingQueue', schema=None) as batch_op:
        batch_op.create_index(batch_op.f('ix_BookcaseItemBorrowingQueue_fk_bookcase_item_uid'), ['fk_bookcase_item_uid'], unique=False)

    op.create_table('Item_Author',
    sa.Column('fk_item_uid', sa.Integer(), nullable=False),
    sa.Column('fk_author_uid', sa.Integer(), nullable=False),
    sa.ForeignKeyConstraint(['fk_author_uid'], ['Author.uid'], name=op.f('fk_Item_Author_fk_author_uid_Author')),
    sa.ForeignKeyConstraint(['fk_item_uid'], ['BookcaseItem.uid'], name=op.f('fk_Item_Author_fk_item_uid_BookcaseItem')),
    sa.PrimaryKeyConstraint('fk_item_uid', 'fk_author_uid', name=op.f('pk_Item_Author'))
    )
    op.create_table('Item_Category',
    sa.Column('fk_item_uid', sa.Integer(), nullable=False),
    sa.Column('fk_category_uid', sa.Integer(), nullable=False),
    sa.ForeignKeyConstraint(['fk_category_uid'], ['Category.uid'], name=op.f('fk_Item_Category_fk_category_uid_Category')),
    sa.ForeignKeyConstraint(['fk_item_uid'], ['BookcaseItem.uid'], name=op.f('fk_Item_Category_fk_item_uid_BookcaseItem')),
    sa.PrimaryKeyConstraint('fk_item_uid', 'fk_category_uid', name=op.f('pk_Item_Category'))
    )
    # ### end Alembic commands ###


def downgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_table('Item_Category')
    op.drop_table('Item_Author')
    with op.batch_alter_table('BookcaseItemBorrowingQueue', schema=None) as batch_op:
        batch_op.drop_index(batch_op.f('ix_BookcaseItemBorrowingQueue_fk_bookcase_item_uid'))

    op.drop_table('BookcaseItemBorrowingQueue')
    with op.batch_alter_table('BookcaseItemBorrowing', schema=None) as batch_op:
        batch_op.drop_index(batch_op.f('ix_BookcaseItemBorrowing_fk_bookcase_item_uid'))

    op.drop_table('BookcaseItemBorrowing')
    with op.batch_alter_table('BookcaseItem', schema=None) as batch_op:
        batch_op.drop_index(batch_op.f('ix_BookcaseItem_name'))
        batch_op.drop_index(batch_op.f('ix_BookcaseItem_isbn'))

    op.drop_table('BookcaseItem')
    op.drop_table('BookcaseShelf')
    with op.batch_alter_table('MediaType', schema=None) as batch_op:
        batch_op.drop_index(batch_op.f('ix_MediaType_name'))

    op.drop_table('MediaType')
    with op.batch_alter_table('Language', schema=None) as batch_op:
        batch_op.drop_index(batch_op.f('ix_Language_name'))
        batch_op.drop_index(batch_op.f('ix_Language_iso639_1_code'))

    op.drop_table('Language')
    with op.batch_alter_table('Category', schema=None) as batch_op:
        batch_op.drop_index(batch_op.f('ix_Category_name'))

    op.drop_table('Category')
    with op.batch_alter_table('Bookcase', schema=None) as batch_op:
        batch_op.drop_index(batch_op.f('ix_Bookcase_name'))

    op.drop_table('Bookcase')
    with op.batch_alter_table('Author', schema=None) as batch_op:
        batch_op.drop_index(batch_op.f('ix_Author_name'))

    op.drop_table('Author')
    # ### end Alembic commands ###
